@@ -184,6 +184,9 @@ static const test_pair_t combining_class_tests_more[] =
   /* Unicode-14.0 character additions */
   {   0x1DFA, 218 },
 
+  /* Unicode-15.0 character additions */
+  {  0x10EFD, 220 },
+
   { 0x111111, 0 }
 };
 
@@ -267,6 +270,9 @@ static const test_pair_t general_category_tests_more[] =
   /* Unicode-14.0 character additions */
   {   0x20C0, HB_UNICODE_GENERAL_CATEGORY_CURRENCY_SYMBOL },
 
+  /* Unicode-15.0 character additions */
+  {   0x0CF3, HB_UNICODE_GENERAL_CATEGORY_SPACING_MARK },
+
   { 0x111111, HB_UNICODE_GENERAL_CATEGORY_UNASSIGNED }
 };
 
@@ -524,6 +530,10 @@ static const test_pair_t script_tests_more[] =
   {  0x16A70, HB_SCRIPT_TANGSA },
   {  0x1E290, HB_SCRIPT_TOTO },
 
+  /* Unicode-15.0 additions */
+  {  0x11F00, HB_SCRIPT_KAWI },
+  {  0x1E4D0, HB_SCRIPT_NAG_MUNDARI },
+
   { 0x111111, HB_SCRIPT_UNKNOWN }
 };
 
